@@ -113,6 +113,7 @@ pub fn create_container(
     location: &Path,
     user: &str,
     command: &str,
+    extra_args: &[String],
     interrupted: &Arc<AtomicBool>,
 ) -> Result<String, Failure> {
     debug!("Creating container from image {}\u{2026}", image.code_str());
@@ -129,6 +130,7 @@ pub fn create_container(
         mount_paths,
         mount_readonly,
         ports,
+        extra_args,
     ));
 
     args.extend(
@@ -425,6 +427,7 @@ pub fn spawn_shell(
     mount_readonly: bool,
     ports: &[String],
     user: &str,
+    extra_args: &[String],
     interrupted: &Arc<AtomicBool>,
 ) -> Result<(), Failure> {
     debug!(
@@ -444,6 +447,7 @@ pub fn spawn_shell(
         mount_paths,
         mount_readonly,
         ports,
+        extra_args,
     ));
 
     args.extend(
@@ -464,6 +468,7 @@ fn container_args(
     mount_paths: &[MappingPath],
     mount_readonly: bool,
     ports: &[String],
+    extra_args: &[String],
 ) -> Vec<String> {
     // Why `--init`? (1) PID 1 is supposed to reap orphaned zombie processes, otherwise they can
     // accumulate. Bash does this, but we run `/bin/sh` in the container, which may or may not be
@@ -476,12 +481,9 @@ fn container_args(
 
     // Environment
     args.extend(
-        environment
-            .iter()
-            .flat_map(|(variable, value)| {
-                vec!["--env".to_owned(), format!("{}={}", variable, value)]
-            })
-            .collect::<Vec<_>>(),
+        environment.iter().flat_map(|(variable, value)| {
+            vec!["--env".to_owned(), format!("{}={}", variable, value)]
+        }),
     );
 
     // Location
@@ -491,30 +493,25 @@ fn container_args(
     ]);
 
     // Mount paths
-    args.extend(
-        mount_paths
-            .iter()
-            .flat_map(|mount_path| {
-                // [ref:mount_paths_no_commas]
-                vec![
-                    "--mount".to_owned(),
-                    if mount_readonly {
-                        format!(
-                            "type=bind,source={},target={},readonly",
-                            source_dir.join(&mount_path.host_path).to_string_lossy(),
-                            location.join(&mount_path.container_path).to_string_lossy(),
-                        )
-                    } else {
-                        format!(
-                            "type=bind,source={},target={}",
-                            source_dir.join(&mount_path.host_path).to_string_lossy(),
-                            location.join(&mount_path.container_path).to_string_lossy(),
-                        )
-                    },
-                ]
-            })
-            .collect::<Vec<_>>(),
-    );
+    args.extend(mount_paths.iter().flat_map(|mount_path| {
+        // [ref:mount_paths_no_commas]
+        vec![
+            "--mount".to_owned(),
+            if mount_readonly {
+                format!(
+                    "type=bind,source={},target={},readonly",
+                    source_dir.join(&mount_path.host_path).to_string_lossy(),
+                    location.join(&mount_path.container_path).to_string_lossy(),
+                )
+            } else {
+                format!(
+                    "type=bind,source={},target={}",
+                    source_dir.join(&mount_path.host_path).to_string_lossy(),
+                    location.join(&mount_path.container_path).to_string_lossy(),
+                )
+            },
+        ]
+    }));
 
     // Ports
     args.extend(
@@ -529,6 +526,9 @@ fn container_args(
             .collect::<Vec<_>>(),
     );
 
+    // User-provided arguments
+    args.extend_from_slice(extra_args);
+
     args
 }
 
